@@ -4,12 +4,12 @@
 # linux distribution.
 
 # If we have lsb_version, use it
-[ -x /usr/bin/lsb_release ] && exec lsb_release -sr
+[ -x /usr/bin/lsb_release -o -x /bin/lsb_release ] && exec lsb_release -sr
 
 # Safe fallback
-for dist in redhat mandrake turbolinux linuxppc
+for dist in redhat mandrake turbolinux linuxppc yellowdog
 do
-	if rpm -q $dist-release &>/dev/null
+	if rpm -q $dist-release >/dev/null 2>&1
 	then
 		rpm -q --queryformat "%{version}\n" $dist-release
 		exit 0
