@@ -1,4 +1,4 @@
-package autoupdate;  # $Date: 2002/10/12 $
+package autoupdate;  # $Date: 2003/03/12 $
 
 use strict;
 # warnings causes problems under 5.005
@@ -10,9 +10,9 @@ BEGIN {
 	use vars qw($VERSION $DEBUG @ISA @EXPORT @EXPORT_OK);
 
 	# set the version for version checking
-	$VERSION     = 1.4.1;
+	$VERSION     = 1.4.5;
 	@ISA         = qw(Exporter);
-	@EXPORT      = qw(&CallRPM &SetArch &SetDistArch &GetArch &GetDistArch
+	@EXPORT      = qw(&CallRPM &CheckRPMVersion &SetArch &SetDistArch &GetArch &GetDistArch
 			&GetAllowedArch &GetAllowedDistArch &TestArch &IsElement
 			&SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &GetLocal
 			&SelectRPMs &LoadRpmBindings &TieRpmDatabase);
@@ -20,13 +20,15 @@ BEGIN {
 }
 END {};
 
-use vars qw($DEBUG $RpmModule $UsePipe $AddInstalled);
+use vars qw($DEBUG $RpmModule $UsePipe $AddInstalled $RpmArrayBug);
 
 $DEBUG = 0;
 $UsePipe = 0;
 $AddInstalled = 0;
 $RpmModule = 0;
-$RPM::err = "unknown error";
+$RPM::err = "unknown rpm error";
+#Some older versions of rpm segfault when the array of a tag is empty
+$RpmArrayBug = 0;
 
 my $RPM = "/bin/rpm";
 my $TmpFile = "autoupdate_$$.tmp";
@@ -54,18 +56,41 @@ my %TAGS = (
 	'OBSOLETEVERSION' => 1 );
 # Build up the query format from %TAGS
 my $QueryFormat= "";
-foreach my $tag (keys %TAGS) {
-	if ($TAGS{$tag}) {
-		$QueryFormat .= "[$tag:%{$tag}\\n]";
+
+if (-x $RPM) {
+	my @RPMOut;
+	unless (CheckRPMVersion("3.0.3") ge 0) {
+		$RpmArrayBug= 1;
+	}
+	if (CallRPM(\@RPMOut, "--querytags") == 0) {
+		foreach my $tag (keys %TAGS) {
+			unless (IsElement($tag, @RPMOut)) {
+				print "AUPM: tag $tag not supported by rpm.\n" if $DEBUG;
+				next;
+			}
+			if ($TAGS{$tag}) {
+				$QueryFormat .= "[$tag:%{$tag}\\n]" unless ($RpmArrayBug);
+			} else {
+			$QueryFormat .= "$tag:%{$tag}\\n";
+			}
+		}
+		if (!IsElement('EPOCH', @RPMOut) and IsElement('SERIAL', @RPMOut)) {
+			$QueryFormat .= "EPOCH:%{SERIAL}\\n";
+		}
+		for my $tag ( qw(PROVIDE OBSOLETE) ) {
+			if (!IsElement("${tag}NAME", @RPMOut) and IsElement("${tag}S", @RPMOut)) {
+				$QueryFormat .= "[${tag}NAME:%{${tag}S}\\n${tag}FLAGS:\\n${tag}VERSION:\\n]" unless ($RpmArrayBug);
+			}
+		}
 	} else {
-		$QueryFormat .= "$tag:%{$tag}\\n";
+		print STDERR "AUPM: Could not call rpm!\n" if $DEBUG;
 	}
 }
 
-
 my $TieRpmDatabase = 0;
 my $Arch;
 my $DistArch;
+my $RPMVersion;
 my $RPM_DB;
 my %LocalRPMs;
 my %LocalRPMsArchs;
@@ -82,7 +107,7 @@ sub CallRPM
 	my $rpmoutput= shift;
 	# If a daemon gets restarted in the post script, the pipe
 	# doesn't get closed until the daemon exists under some
-	# older versions of rpm. 
+	# older versions of rpm. Hence using a pipe is not safe. 
 
 	if ($UsePipe) {
 		pipe(R, W);
@@ -110,7 +135,7 @@ sub CallRPM
 			close(W);
 		}	
 		waitpid($pid,0);
-		my $retval= $? >> 8;
+		my $retval= $?;
 		unless ($UsePipe) {
 			open(R,"< $TmpFile") || die("Could not open: $TmpFile ($!)");
 		}
@@ -138,32 +163,66 @@ sub QueryRPM {
 	} else {
 		$flag = "-qp"
 	}
-	unless ( CallRPM(\@RPMOut, $flag, "--queryformat", $QueryFormat, $item) == 0 ) {
-		$RPM::err= $RPMOut[0];
-		return undef;
+
+	# Initialize arrays
+	my %HDR = ();
+	for my $tag (keys %TAGS) {
+		if ($TAGS{$tag}) {
+			$HDR{$tag} = [];
+		} else {
+			$HDR{$tag} = "";
+		}
 	}
 
-	my %HDR = ();
+	unless ( CallRPM(\@RPMOut, $flag, "--queryformat", $QueryFormat, $item) == 0 ) {
+		$RPM::err= $RPMOut[0];
+		return ();
+	}
 	for my $line (@RPMOut) {
+		$line =~ s/^\(none\)//;
 		next unless ($line =~ /^([A-Z]+):(.*)$/);
 		my ($tag, $key) = ($1, $2);
 		next unless ( defined $TAGS{$tag} );
 		if ($TAGS{$tag}) {
-			if ( defined $HDR{$tag} ) {
-				push(@{$HDR{$tag}}, $key);
-			} else {
-				$HDR{$tag}= [ $key ];
-			}
+			push(@{$HDR{$tag}}, $key);
 		} else {
-			$HDR{$tag} = $key;
+			if ($key eq '(none)') {
+				$HDR{$tag}= "";
+			} else {
+				$HDR{$tag}= $key;
+			}
 		}	
 	}
-	if ($HDR{'EPOCH'} and $HDR{'EPOCH'} eq '(none)') {
-		$HDR{'EPOCH'}= "";
-	}
 	return %HDR;
 }
 
+######################
+# Check for a certain RPM version
+######################
+
+sub CheckRPMVersion {
+	my $ReqVersion= shift;
+
+	unless ($ReqVersion =~ /-/) {
+		$ReqVersion .= "-0";
+	}
+	unless ($RPMVersion) {
+		my @RPMOut;
+		CallRPM(\@RPMOut, "-q", "rpm" );
+		$RPMVersion="0.0-0";
+		foreach my $item (@RPMOut) {
+			if ($item =~ /^rpm-(\d+\..*)$/) {
+				$RPMVersion= $1;
+				last;
+			}
+		}
+	}
+	my $rpml= Header->new("rpm-$RPMVersion.noarch.rpm", "r");
+	my $rpmr= Header->new("rpm-$ReqVersion.noarch.rpm", "r");
+ 
+	return $rpml->compare($rpmr);
+}
+
 ##################################
 # Class for RPM header information
 ##################################
@@ -179,7 +238,7 @@ sub new {
 	my $name = "";
 	my $version = "";
 	my $release = "1";
-	my $serial = "";
+	my $epoch = "";
 	my $arch = "noarch";
 	my $vendor = "";
 	my $type = "r";
@@ -204,19 +263,23 @@ sub new {
 		$arch= $4;
 	}
 	if ($version =~ /^(\d+):(.+)$/) {
-		$serial= $1;
+		$epoch= $1;
 		$version=$2;
 	} elsif ($version =~ /^\(none\):(.+)$/) {
 		$version=$1;
 	}
 
 	$self->{NAME} = $name;
-	$self->{EPOCH} = $serial;
+	$self->{EPOCH} = $epoch;
 	$self->{VERSION} = $version;
 	$self->{RELEASE} = $release;
 	$self->{ARCH} = $arch;
 	$self->{VENDOR} = $vendor;
-	$self->{FILENAME} = "$path$file";
+	if ($type eq 'l') {
+		$self->{FILENAME} = $name . "-" . $version . "-" . $release;
+	} else {
+		$self->{FILENAME} = "$path$file";
+	}
 	$self->{TYPE} = $type;
 	$self->{HDR} = $hdr;
 	bless ($self, $class);
@@ -305,14 +368,10 @@ sub getheader {
 					last if ( $hdr->version eq $self->version and $hdr->release eq $self->release);				}
 			}
 		} else {
-			my %HDR = &autoupdate::QueryRPM("l", $self->name . "-" . $self->version);
-			$hdr= \%HDR;
+			my %HDR = &autoupdate::QueryRPM("l", $self->filename);
+			$hdr= \%HDR if ( $HDR{'NAME'} );
 		}
-		unless ($hdr) {
-			print STDERR "Error: " . $self->rpmname . ": $RPM::err\n";
-			$self->{TYPE} = "r"; # Don't try again
-			return undef;
-		} else {
+		if ($hdr) {
 			push(@LocalTied, $self);
 		}
 	} elsif ($self->{TYPE} eq "f") {
@@ -326,13 +385,19 @@ sub getheader {
 			$hdr = RPM::Header->new( $self->filename );
 		} else {
 			my %HDR = &autoupdate::QueryRPM("f", $self->filename);
-			$hdr= \%HDR;
+			$hdr= \%HDR if ( $HDR{'NAME'} );
 		}
-		unless ($hdr) {
-       		print STDERR "Error: " . $self->filename . ": $RPM::err\n";
-			$self->{TYPE} = "r";
-			return undef;
+	}
+	unless ($hdr) {
+		my $file= $self->filename;
+		$RPM::err =~ s/error:/rpm error:/;
+		if ( $RPM::err =~ /\Q$file/) {
+       			print STDERR "Error: $RPM::err\n";
+		} else {
+       			print STDERR "Error: $file: $RPM::err\n";
 		}
+		$self->{TYPE} = "r"; # Don't try again
+		return undef;
 	}
 	$self->{HDR} = $hdr;
 	foreach my $tag ( qw(NAME EPOCH VERSION RELEASE ARCH VENDOR) ) {
@@ -356,7 +421,7 @@ sub gettag {
 	} else {
 		$tag = uc($tag);
 		my $tmp = $hdr->{$tag};
-		return undef unless ($tmp);
+		return undef unless (defined $tmp);
 		if (ref($tmp) eq "ARRAY") {
 			return @{$tmp};
 		} else {
@@ -369,6 +434,9 @@ sub files {
 	my $self = shift;
 	my ($flags, @files);
 
+	if ($self->{TYPE} eq "r") {
+		return ();
+	}
 	if ($autoupdate::RpmModule==0 or ($autoupdate::RpmModule==3 and $self->{TYPE} eq "l")) {
 		if ($self->{TYPE} eq "l") {
 			$flags= "-q";
@@ -471,28 +539,32 @@ sub compare {
 			return $tmp;
 		}
 	}
-	if ($rpma->{TYPE} eq "f" and !$rpma->{HDR}) {
+	if ($rpma->{TYPE} ne "r" and !$rpma->{HDR}) {
 		$rpma->getheader();
 	}
-	if ($rpmb->{TYPE} eq "f" and !$rpmb->{HDR}) {
+	if ($rpmb->{TYPE} ne "r" and !$rpmb->{HDR}) {
 		$rpmb->getheader();
 	}
 
 	$a = $rpma->{EPOCH};
+	$a = 0 unless ($a or $rpma->{TYPE} eq "r");
 	$b = $rpmb->{EPOCH};
-	if ($a and $b and $a ne $b) { return ($a <=> $b)};
-
+	$b = 0 unless ($b or $rpmb->{TYPE} eq "r");
+	if ($a ne "" and $b ne "" and $a ne $b) { return ($a <=> $b)};
 	$a = $rpma->{VERSION};
 	$b = $rpmb->{VERSION};
 
 	if ($a eq $b) {
 		#Versions are equal, need to compare the release
-		#We only take the numeric part if present
 		my $ra= $rpma->{RELEASE};
 		my $rb= $rpmb->{RELEASE};
 		# Save some time
 		if ($ra eq $rb) { return 0 };
 
+		# Rename 1fix is newer than 1
+		if ( $ra =~ /^${rb}fix(ed)?$/i) { return 1 };
+		if ( $rb =~ /^${ra}fix(ed)?$/i) { return -1 };
+		#We only take the numeric part if present
 		if ( $ra=~/^(.*\d)[^.\d]*$/ ) {
 			$a=$1;
 			} else {
@@ -655,6 +727,8 @@ sub SetArch {
 				}
 			close(FILE);
 			}
+		} elsif ($Arch =~ /00.+00/) { # AIX returns a long system id string like "000734CF4C00"
+			$Arch= "ppc";
 		}
 	}
 }
@@ -908,7 +982,7 @@ sub TieRpmDatabase {
 		if ($RpmModule == 1) {
 			untie(%RPM);
 		} elsif ($RpmModule == 2) {
-			$RPM_DB->close_rpm_db();
+			#$RPM_DB->close_rpm_db();
 			$RPM_DB= undef;
 		}
 		return 0;
@@ -1176,7 +1250,7 @@ be used to specify the type.
  "r(emote)" the rpm corresponds to a remote file which cannot
          be queried
 
-The string shouold be of the form "[/path/]name-[serial:]version-release.arch.rpm".
+The string shouold be of the form "[/path/]name-[epoch:]version-release.arch.rpm".
 
 Methods implemented are
 
@@ -1204,7 +1278,7 @@ Each object contains the following date:
  $rpm->{RELEASE}  release tag of the rpm
  $rpm->{ARCH}     arch tag of the rpm
  $rpm->{FILENAME} the full file name
- $rpm->{TYPE}     the type (f|l|c)
+ $rpm->{TYPE}     the type (f|l|r)
  $rpm->{HDR}      pointer to the header object returned by the rpm bindings
 
 =head2 Functions
@@ -1213,6 +1287,7 @@ First of all there the following functions related to architectures defined:
 
  &CallRPM(\@Out, @Args)    Calls rpm with the given arguments and saves the output
                            in @Out. Returns the exit code of rpm.
+ &CheckRPMVersion($ver)    Check if rpm is newer than $ver.
 
  &SetArch([$string])       Sets the architecture to the given value or (if none is
                            given) tries to determine it from the current machine.
